"""
This module is based on a rox module (LGPL):

http://cvs.sourceforge.net/viewcvs.py/rox/ROX-Lib2/python/rox/basedir.py?rev=1.9&view=log

The freedesktop.org Base Directory specification provides a way for
applications to locate shared data and configuration:

    http://standards.freedesktop.org/basedir-spec/

(based on version 0.6)

This module can be used to load and save from and to these directories.

Typical usage:

    from rox import basedir

    for dir in basedir.load_config_paths('mydomain.org', 'MyProg', 'Options'):
        print "Load settings from", dir

    dir = basedir.save_config_path('mydomain.org', 'MyProg')
    print >>file(os.path.join(dir, 'Options'), 'w'), "foo=2"

Note: see the rox.Options module for a higher-level API for managing options.
"""

from __future__ import generators
import os
import sys
import ctypes

# we must chech which os we are using to ensure that the correct
# paths are set
if sys.platform == "win32":

    def get_special_folders():

        # Routine to grab all the Windows Special Folders locations.
        # If successful, returns dictionary
        # of shell folder locations indexed on Windows keyword for each;
        # otherwise, returns an empty dictionary.
        special_folders = {}

        dll = ctypes.windll.shell32
        buf = ctypes.create_string_buffer(300)
        # 0x011C = C:\Users\<username>\AppData\Local
        # 0x0028 = C:\Users\<username>
        if dll.SHGetSpecialFolderPathA(None, buf, 0x0028, False):
            special_folders['Personal'] = buf.value
        if dll.SHGetSpecialFolderPathA(None, buf, 0x011C, False):
            special_folders['Local AppData'] = buf.value
            special_folders['AppData'] = os.path.dirname(buf.value)

        return special_folders


    special_folders = get_special_folders()

    home_path = special_folders['Personal']
    app_local_data_path = special_folders['Local AppData']
    app_global_data_path = special_folders['AppData']

    # use the non romain app data
    xdg_data_home = os.environ.get('XDG_DATA_HOME',
        os.path.join(app_local_data_path, 'xdg'))

    # we will use ; as the spearator because is the char used
    # in the windows env variables
    xdg_data_dirs = os.environ.get('XDG_DATA_DIRS',
        '{0};{1}'.format(app_local_data_path,
        app_global_data_path)).split(';')

    # we will return the roaming data wich is as close as we get in windows
    # regarding caching.
    xdg_cache_home = os.environ.get('XDG_CACHE_HOME',
        os.path.join(xdg_data_home, 'cache'))

    # point to the not roaming app data for the user
    xdg_config_home = os.environ.get('XDG_CONFIG_HOME',
        app_local_data_path)

    xdg_config_dirs = [xdg_config_home] + \
        os.environ.get('XDG_CONFIG_DIRS',
        os.path.join(app_global_data_path, 'xdg')).split(':')

else:
    _home = os.environ.get('HOME', '/')
    xdg_data_home = os.environ.get('XDG_DATA_HOME',
        os.path.join(_home, '.local', 'share'))

    xdg_data_dirs = [xdg_data_home] + \
        os.environ.get('XDG_DATA_DIRS',
            '/usr/local/share:/usr/share').split(':')

    xdg_config_home = os.environ.get('XDG_CONFIG_HOME',
        os.path.join(_home, '.config'))

    xdg_config_dirs = [xdg_config_home] + \
        os.environ.get('XDG_CONFIG_DIRS', '/etc/xdg').split(':')

    xdg_cache_home = os.environ.get('XDG_CACHE_HOME',
        os.path.join(_home, '.cache'))

xdg_data_dirs = filter(lambda x: x, xdg_data_dirs)
xdg_config_dirs = filter(lambda x: x, xdg_config_dirs)


def save_config_path(*resource):
    """Ensure $XDG_CONFIG_HOME/<resource>/ exists, and return its path.
    'resource' should normally be the name of your application. Use this
    when SAVING configuration settings. Use the xdg_config_dirs variable
    for loading."""
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    path = os.path.join(xdg_config_home, resource)
    if not os.path.isdir(path):
        os.makedirs(path, 0700)
    return path


def save_data_path(*resource):
    """Ensure $XDG_DATA_HOME/<resource>/ exists, and return its path.
    'resource' is the name of some shared resource. Use this when updating
    a shared (between programs) database. Use the xdg_data_dirs variable
    for loading."""
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    path = os.path.join(xdg_data_home, resource)
    if not os.path.isdir(path):
        os.makedirs(path)
    return path


def load_config_paths(*resource):
    """Returns an iterator which gives each directory named 'resource'
    in the configuration search path. Information provided by earlier
    directories should take precedence over later ones (ie, the user's
    config dir comes first)."""
    resource = os.path.join(*resource)
    for config_dir in xdg_config_dirs:
        path = os.path.join(config_dir, resource)
        if os.path.exists(path):
            yield path


def load_first_config(*resource):
    """Returns the first result from load_config_paths,
    or None if there is nothing to load."""
    for x in load_config_paths(*resource):
        return x
    return None


def load_data_paths(*resource):
    """Returns an iterator which gives each directory named 'resource' in the
    shared data search path. Information provided by earlier directories should
    take precedence over later ones."""
    resource = os.path.join(*resource)
    for data_dir in xdg_data_dirs:
        path = os.path.join(data_dir, resource)
        if os.path.exists(path):
            yield path
